package ru.narfu.kozalo.des


infix fun Int.sh(pos: Int): Int {
    return if (pos >= 0) {
        this shr pos
    } else {
        this shl -pos
    }
}


infix fun ByteArray.shl(offset: Int): ByteArray = when {
    offset > 8 -> (this shl (offset - 8)) shlImpl 8
    else -> this shlImpl offset
}


private infix fun ByteArray.shlImpl(offset: Int): ByteArray {
    assert(offset <= 8)

    val restLength = 8 - offset
    val mask = (0xFF shl restLength) and 0xFF
    var lastBits = 0
    val newByteArray = this.reversed().map { it.toInt() }.map { byte ->
        val newByte = (byte shl offset) or lastBits
        lastBits = (byte and mask) shr restLength
        newByte
    }.reversed().toMutableList()
    val lastIndex = newByteArray.size - 1
    newByteArray[lastIndex] = newByteArray[lastIndex] or lastBits
    return newByteArray.map { it.toByte() }.toByteArray()
}


infix fun ByteArray.shr(offset: Int): ByteArray = when {
    offset > 8 -> (this shr (offset - 8)) shrImpl 8
    else -> this shrImpl offset
}


private infix fun ByteArray.shrImpl(offset: Int): ByteArray {
    assert(offset <= 8)

    val mask = 0xFF shr (8 - offset)
    var firstBits = 0
    val newBY = this.map { it.toInt() }.map { byte ->
        val newByte = ((byte and 0xFF) shr offset) or firstBits
        firstBits = (byte and mask) shl (8 - offset)
        newByte
    }.toMutableList()
    newBY[0] = newBY[0] or firstBits
    return newBY.map { it.toByte() }.toByteArray()
}
